# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: translationContainerGRPC.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1etranslationContainerGRPC.proto\x12\x1ctranslationContainerServices\"\xd5\x01\n&TrCustomMessageToMythicC2FormatMessage\x12 \n\x18TranslationContainerName\x18\x01 \x01(\t\x12\x0e\n\x06\x43\x32Name\x18\x02 \x01(\t\x12\x0f\n\x07Message\x18\x03 \x01(\x0c\x12\x0c\n\x04UUID\x18\x04 \x01(\t\x12\x16\n\x0eMythicEncrypts\x18\x05 \x01(\x08\x12\x42\n\nCryptoKeys\x18\x06 \x03(\x0b\x32..translationContainerServices.CryptoKeysFormat\"A\n\x10\x43ryptoKeysFormat\x12\x0e\n\x06\x45ncKey\x18\x01 \x01(\x0c\x12\x0e\n\x06\x44\x65\x63Key\x18\x02 \x01(\x0c\x12\r\n\x05Value\x18\x03 \x01(\t\"\x83\x01\n.TrCustomMessageToMythicC2FormatMessageResponse\x12\x0f\n\x07Success\x18\x01 \x01(\x08\x12\r\n\x05\x45rror\x18\x02 \x01(\t\x12\x0f\n\x07Message\x18\x03 \x01(\x0c\x12 \n\x18TranslationContainerName\x18\x04 \x01(\t\"\xd5\x01\n&TrMythicC2ToCustomMessageFormatMessage\x12 \n\x18TranslationContainerName\x18\x01 \x01(\t\x12\x0e\n\x06\x43\x32Name\x18\x02 \x01(\t\x12\x0f\n\x07Message\x18\x03 \x01(\x0c\x12\x0c\n\x04UUID\x18\x04 \x01(\t\x12\x16\n\x0eMythicEncrypts\x18\x05 \x01(\x08\x12\x42\n\nCryptoKeys\x18\x06 \x03(\x0b\x32..translationContainerServices.CryptoKeysFormat\"\x83\x01\n.TrMythicC2ToCustomMessageFormatMessageResponse\x12\x0f\n\x07Success\x18\x01 \x01(\x08\x12\r\n\x05\x45rror\x18\x02 \x01(\t\x12\x0f\n\x07Message\x18\x03 \x01(\x0c\x12 \n\x18TranslationContainerName\x18\x04 \x01(\t\"\x86\x01\n\x1fTrGenerateEncryptionKeysMessage\x12 \n\x18TranslationContainerName\x18\x01 \x01(\t\x12\x0e\n\x06\x43\x32Name\x18\x02 \x01(\t\x12\x18\n\x10\x43ryptoParamValue\x18\x03 \x01(\t\x12\x17\n\x0f\x43ryptoParamName\x18\x04 \x01(\t\"\x99\x01\n\'TrGenerateEncryptionKeysMessageResponse\x12\x0f\n\x07Success\x18\x01 \x01(\x08\x12\r\n\x05\x45rror\x18\x02 \x01(\t\x12\x15\n\rEncryptionKey\x18\x03 \x01(\x0c\x12\x15\n\rDecryptionKey\x18\x04 \x01(\x0c\x12 \n\x18TranslationContainerName\x18\x05 \x01(\t2\xbd\x04\n\x14TranslationContainer\x12\xbd\x01\n!TranslateFromCustomToMythicFormat\x12L.translationContainerServices.TrCustomMessageToMythicC2FormatMessageResponse\x1a\x44.translationContainerServices.TrCustomMessageToMythicC2FormatMessage\"\x00(\x01\x30\x01\x12\xbd\x01\n!TranslateFromMythicToCustomFormat\x12L.translationContainerServices.TrMythicC2ToCustomMessageFormatMessageResponse\x1a\x44.translationContainerServices.TrMythicC2ToCustomMessageFormatMessage\"\x00(\x01\x30\x01\x12\xa4\x01\n\x16GenerateEncryptionKeys\x12\x45.translationContainerServices.TrGenerateEncryptionKeysMessageResponse\x1a=.translationContainerServices.TrGenerateEncryptionKeysMessage\"\x00(\x01\x30\x01\x42/Z-github.com/its-a-feature/Mythic/grpc/servicesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'translationContainerGRPC_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z-github.com/its-a-feature/Mythic/grpc/services'
  _TRCUSTOMMESSAGETOMYTHICC2FORMATMESSAGE._serialized_start=65
  _TRCUSTOMMESSAGETOMYTHICC2FORMATMESSAGE._serialized_end=278
  _CRYPTOKEYSFORMAT._serialized_start=280
  _CRYPTOKEYSFORMAT._serialized_end=345
  _TRCUSTOMMESSAGETOMYTHICC2FORMATMESSAGERESPONSE._serialized_start=348
  _TRCUSTOMMESSAGETOMYTHICC2FORMATMESSAGERESPONSE._serialized_end=479
  _TRMYTHICC2TOCUSTOMMESSAGEFORMATMESSAGE._serialized_start=482
  _TRMYTHICC2TOCUSTOMMESSAGEFORMATMESSAGE._serialized_end=695
  _TRMYTHICC2TOCUSTOMMESSAGEFORMATMESSAGERESPONSE._serialized_start=698
  _TRMYTHICC2TOCUSTOMMESSAGEFORMATMESSAGERESPONSE._serialized_end=829
  _TRGENERATEENCRYPTIONKEYSMESSAGE._serialized_start=832
  _TRGENERATEENCRYPTIONKEYSMESSAGE._serialized_end=966
  _TRGENERATEENCRYPTIONKEYSMESSAGERESPONSE._serialized_start=969
  _TRGENERATEENCRYPTIONKEYSMESSAGERESPONSE._serialized_end=1122
  _TRANSLATIONCONTAINER._serialized_start=1125
  _TRANSLATIONCONTAINER._serialized_end=1698
# @@protoc_insertion_point(module_scope)
